import { traverseGuiControls } from '../gui-controls.js';
import { buildGraphNodeId } from '../compile-dsp-graph/to-dsp-graph.js';
import { builders } from '../nodes/nodes/controls-float.js';

const FLOAT_CONTROL_TYPES = [...Object.keys(builders), 'floatatom'];
const collectIoMessageReceiversFromSendNodes = (pdJson, graph) => {
    const rootPatch = pdJson.patches[pdJson.rootPatchId];
    const messageReceivers = {};
    Object.values(rootPatch.nodes).forEach((pdNode) => {
        const nodeId = buildGraphNodeId(rootPatch.id, pdNode.id);
        const node = graph[nodeId];
        if (pdNode.type === 'send' &&
            // Important because some nodes are deleted at dsp-graph compilation.
            // and if we declare messageReceivers for them it will cause error.
            // TODO : maybe the compiler should detect this instead of doing it here ?
            !!node) {
            if (!messageReceivers[nodeId]) {
                const layout = pdNode.layout || {};
                const metadata = {
                    group: 'send',
                    name: node.args.busName,
                    position: layout.x !== undefined && layout.y !== undefined
                        ? [layout.x, layout.y]
                        : undefined,
                };
                messageReceivers[nodeId] = {
                    portletIds: [],
                    metadata: metadata,
                };
            }
            messageReceivers[nodeId].portletIds.push('0');
        }
    });
    return messageReceivers;
};
const collectIoMessageReceiversFromGui = (controls, graph) => {
    const messageReceivers = {};
    traverseGuiControls(controls, (control) => {
        const portletId = '0';
        const nodeId = buildGraphNodeId(control.patch.id, control.node.id);
        if (!messageReceivers[nodeId]) {
            const node = graph[nodeId];
            // Important because some nodes are deleted at dsp-graph compilation.
            // and if we declare messageReceivers for them it will cause error.
            // TODO : maybe the compiler should detect this instead of doing it here ?
            if (!node) {
                return;
            }
            const layout = control.node.layout || {};
            let metadata = {
                group: 'control',
                type: control.node.type,
                label: layout.label,
                position: layout.x !== undefined && layout.y !== undefined
                    ? [layout.x, layout.y]
                    : undefined,
            };
            if (FLOAT_CONTROL_TYPES.includes(control.node.type)) {
                metadata = {
                    ...metadata,
                    group: 'control:float',
                    initValue: node.args.initValue,
                    minValue: node.args.minValue,
                    maxValue: node.args.maxValue,
                };
            }
            messageReceivers[nodeId] = {
                portletIds: [],
                metadata: metadata,
            };
        }
        messageReceivers[nodeId].portletIds.push(portletId);
    });
    return messageReceivers;
};

export { collectIoMessageReceiversFromGui, collectIoMessageReceiversFromSendNodes };
