import { mtof, ftom } from '../global-code/funcs.js';
import { AnonFunc, Var, ConstVar } from '../../../../WebPd_compiler/dist/src/ast/declare.js';
import '../../../../WebPd_compiler/dist/src/compile/precompile/proxies.js';
import '../../../../WebPd_compiler/dist/src/stdlib/fs.js';
import '../../../../WebPd_compiler/dist/src/compile/precompile/nodes.js';
import '../../../../WebPd_compiler/dist/src/compile/proxies.js';

// ------------------------------- node builder ------------------------------ //
const builder = {
    translateArgs: () => ({}),
    build: () => ({
        inlets: {
            '0': { type: 'message', id: '0' },
        },
        outlets: {
            '0': { type: 'message', id: '0' },
        },
    }),
};
const makeNodeImplementation = ({ operationCode, dependencies = [], }) => {
    // ------------------------------- node implementation ------------------------------ //
    return {
        messageReceivers: ({ snds }) => ({
            '0': AnonFunc([Var('Message', 'm')]) `
                if (msg_isMatching(m, [MSG_FLOAT_TOKEN])) {
                    ${ConstVar('Float', 'value', 'msg_readFloatToken(m, 0)')}
                    ${snds.$0}(msg_floats([${operationCode}]))
                    return
                }
            `
        }),
        dependencies
    };
};
// ------------------------------------------------------------------- //
const nodeImplementations = {
    'abs': makeNodeImplementation({ operationCode: `Math.abs(value)` }),
    'wrap': makeNodeImplementation({ operationCode: `(1 + (value % 1)) % 1` }),
    'cos': makeNodeImplementation({ operationCode: `Math.cos(value)` }),
    'sqrt': makeNodeImplementation({ operationCode: `value >= 0 ? Math.pow(value, 0.5): 0` }),
    'mtof': makeNodeImplementation({ operationCode: `mtof(value)`, dependencies: [mtof] }),
    'ftom': makeNodeImplementation({ operationCode: `ftom(value)`, dependencies: [ftom] }),
    'rmstodb': makeNodeImplementation({ operationCode: `value <= 0 ? 0 : 20 * Math.log(value) / Math.LN10 + 100` }),
    'dbtorms': makeNodeImplementation({ operationCode: `value <= 0 ? 0 : Math.exp(Math.LN10 * (value - 100) / 20)` }),
    'powtodb': makeNodeImplementation({ operationCode: `value <= 0 ? 0 : 10 * Math.log(value) / Math.LN10 + 100` }),
    'dbtopow': makeNodeImplementation({ operationCode: `value <= 0 ? 0 : Math.exp(Math.LN10 * (value - 100) / 10)` }),
    // Implement vu as a noop
    'vu': makeNodeImplementation({ operationCode: `value` }),
};
const builders = {
    'abs': builder,
    'cos': builder,
    'wrap': builder,
    'sqrt': builder,
    'mtof': builder,
    'ftom': builder,
    'rmstodb': builder,
    'dbtorms': builder,
    'powtodb': builder,
    'dbtopow': builder,
    'vu': builder,
};

export { builders, nodeImplementations };
