import { Class, Var, AnonFunc, ConstVar } from '../../../../WebPd_compiler/dist/src/ast/declare.js';
import { mapArray } from '../../../../WebPd_compiler/dist/src/functional-helpers.js';
import '../../../../WebPd_compiler/dist/src/compile/precompile/proxies.js';
import '../../../../WebPd_compiler/dist/src/stdlib/fs.js';
import '../../../../WebPd_compiler/dist/src/compile/precompile/nodes.js';
import '../../../../WebPd_compiler/dist/src/compile/proxies.js';
import { bangUtils } from '../global-code/core.js';
import { resolveTypeArgumentAlias, assertTypeArgument, messageTokenToString, messageTokenToFloat } from '../type-arguments.js';

/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
// ------------------------------- node builder ------------------------------ //
const builder = {
    translateArgs: ({ args }) => ({
        typeArguments: (args.length ? args : ['float', 'float']).map(resolveTypeArgumentAlias)
            // Not sure why but 'bang' is supported as a creation argument, 
            // but turned into a float.
            .map(typeArg => {
            // Assign default value
            typeArg = typeArg || 'float';
            if (typeArg === 'bang') {
                typeArg = 'float';
            }
            if (typeof typeArg === 'number') {
                return typeArg;
            }
            else if (!['float', 'symbol'].includes(typeArg)) {
                throw new Error(`${typeArg} not supported (yet)`);
            }
            return typeArg;
        })
            .map(value => [
            typeof value === 'number' ? 'float' : assertTypeArgument(value),
            typeof value === 'number' ? value : value === 'float' ? 0 : 'symbol'
        ]),
    }),
    build: ({ typeArguments }) => ({
        inlets: mapArray(typeArguments, (_, i) => [`${i}`, { type: 'message', id: `${i}` }]),
        outlets: {
            '0': { type: 'message', id: '0' },
        },
    }),
};
// ------------------------------- node implementation ------------------------------ //
const nodeImplementation = {
    state: ({ node: { args }, ns }) => Class(ns.State, [
        Var('Array<Float>', 'floatValues', `[${args.typeArguments.map(([typeArg, defaultValue]) => typeArg === 'float' ? defaultValue : 0).join(',')}]`),
        Var('Array<string>', 'stringValues', `[${args.typeArguments.map(([typeArg, defaultValue]) => typeArg === 'symbol' ? `"${defaultValue}"` : '""').join(',')}]`),
    ]),
    messageReceivers: ({ snds, state, node }) => ({
        '0': AnonFunc([Var('Message', 'm')]) `
            if (!msg_isBang(m)) {
                for (${Var('Int', 'i', '0')}; i < msg_getLength(m); i++) {
                    ${state}.stringValues[i] = messageTokenToString(m, i)
                    ${state}.floatValues[i] = messageTokenToFloat(m, i)
                }
            }
    
            ${ConstVar('MessageTemplate', 'template', `[${node.args.typeArguments.map(([typeArg], i) => typeArg === 'symbol' ?
            `MSG_STRING_TOKEN, ${state}.stringValues[${i}].length`
            : `MSG_FLOAT_TOKEN`).join(',')}]`)}
    
            ${ConstVar('Message', 'messageOut', 'msg_create(template)')}
    
            ${node.args.typeArguments.map(([typeArg], i) => typeArg === 'symbol' ?
            `msg_writeStringToken(messageOut, ${i}, ${state}.stringValues[${i}])`
            : `msg_writeFloatToken(messageOut, ${i}, ${state}.floatValues[${i}])`)}
    
            ${snds[0]}(messageOut)
            return
        `,
        ...mapArray(node.args.typeArguments.slice(1), ([typeArg], i) => {
            if (typeArg === 'symbol') {
                return [
                    `${i + 1}`,
                    AnonFunc([Var('Message', 'm')]) `
                        ${state}.stringValues[${i + 1}] = messageTokenToString(m, 0)
                        return
                    `
                ];
            }
            else if (typeArg === 'float') {
                return [
                    `${i + 1}`,
                    AnonFunc([Var('Message', 'm')]) `
                        ${state}.floatValues[${i + 1}] = messageTokenToFloat(m, 0)
                        return
                    `
                ];
            }
            else {
                throw new Error(`Unsupported type argument ${typeArg}`);
            }
        }),
    }),
    dependencies: [
        messageTokenToString,
        messageTokenToFloat,
        bangUtils,
    ]
};

export { builder, nodeImplementation };
