import { Class, Var, AnonFunc, ast, Sequence, Func, ConstVar } from '../../../../WebPd_compiler/dist/src/ast/declare.js';
import { mapArray, countTo } from '../../../../WebPd_compiler/dist/src/functional-helpers.js';
import '../../../../WebPd_compiler/dist/src/compile/precompile/proxies.js';
import { fsReadSoundStream } from '../../../../WebPd_compiler/dist/src/stdlib/fs.js';
import '../../../../WebPd_compiler/dist/src/compile/precompile/nodes.js';
import '../../../../WebPd_compiler/dist/src/compile/proxies.js';
import { assertOptionalNumber } from '../validation.js';
import { bangUtils, stringMsgUtils } from '../global-code/core.js';
import { parseSoundFileOpenOpts, parseReadWriteFsOpts } from '../global-code/fs.js';

/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
// TODO : check the real state machine of readsf
//      - what happens when start / stopping / start stream ?
//      - what happens when stream ended and starting again ?
//      - etc ...
// TODO : second arg : "buffer channel size" not implemented
// TODO : implement raw
// ------------------------------- node builder ------------------------------ //
const builder = {
    translateArgs: (pdNode) => ({
        channelCount: assertOptionalNumber(pdNode.args[0]) || 1,
    }),
    build: (nodeArgs) => ({
        inlets: {
            '0': { type: 'message', id: '0' },
        },
        outlets: {
            ...mapArray(countTo(nodeArgs.channelCount), (channel) => [`${channel}`, { type: 'signal', id: `${channel}` }]),
            [`${nodeArgs.channelCount}`]: {
                type: 'message',
                id: `${nodeArgs.channelCount}`,
            }
        },
    })
};
// ------------------------------ node implementations ------------------------------ //
const nodeImplementation = {
    flags: {
        alphaName: 'readsf_t',
    },
    state: ({ ns }) => Class(ns.State, [
        Var('Array<buf_SoundBuffer>', 'buffers', '[]'),
        Var('fs_OperationId', 'streamOperationId', -1),
        Var('Int', 'readingStatus', 0),
    ]),
    messageReceivers: ({ ns, node, state, }) => ({
        '0': AnonFunc([Var('Message', 'm')]) `
            if (msg_getLength(m) >= 2) {
                if (msg_isStringToken(m, 0) 
                    && msg_readStringToken(m, 0) === 'open'
                ) {
                    ${ns.openStream}(
                        ${state},
                        m,
                        ${node.args.channelCount},
                        ${AnonFunc() `
                            ${state}.streamOperationId = -1
                            if (${state}.readingStatus === 1) {
                                ${state}.readingStatus = 2
                            } else {
                                ${state}.readingStatus = 3
                            }
                        `}
                    )
                    return
                }
    
            } else if (msg_isMatching(m, [MSG_FLOAT_TOKEN])) {
                if (msg_readFloatToken(m, 0) === 0) {
                    ${state}.readingStatus = 3
                    return
    
                } else {
                    if (${state}.streamOperationId !== -1) {
                        ${state}.readingStatus = 1
                    } else {
                        console.log('[readsf~] start requested without prior open')
                    }
                    return
    
                }
                
            } else if (msg_isAction(m, 'print')) {
                console.log('[readsf~] reading = ' + ${state}.readingStatus.toString())
                return
            }
        `,
    }),
    dsp: ({ state, snds, outs, node: { args: { channelCount }, }, }) => ast `
        switch(${state}.readingStatus) {
            case 1: 
                ${countTo(channelCount).map((i) => `${outs[i]} = buf_pullSample(${state}.buffers[${i}])`)}
                break
                
            case 2: 
                ${countTo(channelCount).map((i) => `${outs[i]} = buf_pullSample(${state}.buffers[${i}])`)}
                if (${state}.buffers[0].pullAvailableLength === 0) {
                    ${snds[channelCount]}(msg_bang())
                    ${state}.readingStatus = 3
                }
                break
    
            case 3: 
                ${countTo(channelCount).map((i) => `${outs[i]} = 0`)}
                ${state}.readingStatus = 0
                break
        }
    `,
    core: ({ ns, globs }) => Sequence([
        Func(ns.openStream, [
            Var(ns.State, 'state'),
            Var('Message', 'm'),
            Var('Int', 'channelCount'),
            Var('fs_OperationCallback', 'onStreamClose'),
        ], 'void') `
                if (state.streamOperationId !== -1) {
                    state.readingStatus = 3
                    fs_closeSoundStream(state.streamOperationId, FS_OPERATION_SUCCESS)
                }
        
                ${ConstVar('fs_SoundInfo', 'soundInfo', `{
                    channelCount,
                    sampleRate: toInt(${globs.sampleRate}),
                    bitDepth: 32,
                    encodingFormat: '',
                    endianness: '',
                    extraOptions: '',
                }`)}
                ${ConstVar('Set<Int>', 'unhandledOptions', `parseSoundFileOpenOpts(
                    m,
                    soundInfo,
                )`)}
                ${ConstVar('string', 'url', `parseReadWriteFsOpts(
                    m,
                    soundInfo,
                    unhandledOptions
                )`)}
                if (url.length === 0) {
                    return
                }
                state.streamOperationId = fs_openSoundReadStream(
                    url,
                    soundInfo,
                    onStreamClose,
                )
                state.buffers = _FS_SOUND_STREAM_BUFFERS.get(state.streamOperationId)
            `
    ]),
    dependencies: [
        parseSoundFileOpenOpts,
        parseReadWriteFsOpts,
        bangUtils,
        stringMsgUtils,
        fsReadSoundStream,
    ],
};

export { builder, nodeImplementation };
