import { assertOptionalString } from '../validation.js';
import { messageBuses } from '../global-code/buses.js';
import { coldStringInlet } from '../standard-message-receivers.js';
import { Class, Var, AnonFunc, ast } from '../../../../WebPd_compiler/dist/src/ast/declare.js';
import '../../../../WebPd_compiler/dist/src/compile/precompile/proxies.js';
import '../../../../WebPd_compiler/dist/src/stdlib/fs.js';
import '../../../../WebPd_compiler/dist/src/compile/precompile/nodes.js';
import '../../../../WebPd_compiler/dist/src/compile/proxies.js';

/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
// ------------------------------- node builder ------------------------------ //
const builderSend = {
    translateArgs: (pdNode) => ({
        busName: assertOptionalString(pdNode.args[0]) || '',
    }),
    build: () => ({
        inlets: {
            '0': { type: 'message', id: '0' },
            '1': { type: 'message', id: '1' },
        },
        outlets: {},
    }),
};
const builderReceive = {
    translateArgs: (pdNode) => ({
        busName: assertOptionalString(pdNode.args[0]) || '',
    }),
    build: () => ({
        inlets: {},
        outlets: {
            '0': { type: 'message', id: '0' },
        },
        isPushingMessages: true
    }),
};
// -------------------------------- node implementation - send ----------------------------------- //
const nodeImplementationSend = {
    state: ({ node: { args }, ns }) => Class(ns.State, [
        Var('string', 'busName', `"${args.busName}"`),
    ]),
    messageReceivers: ({ state, }) => ({
        '0': AnonFunc([Var('Message', 'm')]) `
            msgBusPublish(${state}.busName, m)
            return
        `,
        '1': coldStringInlet(`${state}.busName`)
    }),
    dependencies: [
        messageBuses,
    ],
};
// -------------------------------- node implementation - receive ----------------------------------- //
const nodeImplementationReceive = {
    initialization: ({ node: { args }, snds }) => ast `
            msgBusSubscribe("${args.busName}", ${snds.$0})
        `,
    dependencies: [
        messageBuses,
    ],
};
// ------------------------------------------------------------------------ //
const builders = {
    send: builderSend,
    receive: builderReceive,
};
const nodeImplementations = {
    send: nodeImplementationSend,
    receive: nodeImplementationReceive,
};

export { builders, nodeImplementations };
