import { assertOptionalString } from '../validation.js';
import { bangUtils } from '../global-code/core.js';
import { messageBuses } from '../global-code/buses.js';
import { Class, Var, AnonFunc } from '../../../../WebPd_compiler/dist/src/ast/declare.js';
import '../../../../WebPd_compiler/dist/src/compile/precompile/proxies.js';
import '../../../../WebPd_compiler/dist/src/stdlib/fs.js';
import '../../../../WebPd_compiler/dist/src/compile/precompile/nodes.js';
import '../../../../WebPd_compiler/dist/src/compile/proxies.js';

/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
// TODO: proper support for $ args
// TODO: simple number - shortcut for float
// ------------------------------- node builder ------------------------------ //
const builder = {
    translateArgs: (pdNode) => ({
        value: assertOptionalString(pdNode.args[0]) || '',
    }),
    build: () => ({
        inlets: {
            '0': { type: 'message', id: '0' },
            '1': { type: 'message', id: '1' },
        },
        outlets: {
            '0': { type: 'message', id: '0' },
        },
        isPushingMessages: true,
    }),
};
// ------------------------------- node implementation ------------------------------ //
const nodeImplementation = {
    state: ({ node: { args }, ns }) => Class(ns.State, [
        Var('string', 'value', `"${args.value}"`)
    ]),
    messageReceivers: ({ snds, state, }) => ({
        '0': AnonFunc([Var('Message', 'm')]) `
            if (msg_isBang(m)) {
                ${snds.$0}(msg_strings([${state}.value]))
                return
    
            } else if (msg_isMatching(m, [MSG_STRING_TOKEN])) {
                ${state}.value = msg_readStringToken(m, 0)
                ${snds.$0}(msg_strings([${state}.value]))
                return
    
            }
        `,
        '1': AnonFunc([Var('Message', 'm')]) `
            if (msg_isMatching(m, [MSG_STRING_TOKEN])) {
                ${state}.value = msg_readStringToken(m, 0)
                return 
            }
        `,
    }),
    dependencies: [
        bangUtils,
        messageBuses,
    ]
};

export { builder, nodeImplementation };
