import { ValidationError } from './validation.js';
import { Func, Var, ConstVar } from '../../../WebPd_compiler/dist/src/ast/declare.js';
import '../../../WebPd_compiler/dist/src/compile/precompile/proxies.js';
import '../../../WebPd_compiler/dist/src/stdlib/fs.js';
import '../../../WebPd_compiler/dist/src/compile/precompile/nodes.js';
import '../../../WebPd_compiler/dist/src/compile/proxies.js';

const TYPE_ARGUMENTS = [
    'float',
    'bang',
    'symbol',
    'list',
    'anything',
];
const resolveTypeArgumentAlias = (value) => {
    switch (value) {
        case 'f':
            return 'float';
        case 'b':
            return 'bang';
        case 's':
            return 'symbol';
        case 'l':
            return 'list';
        case 'a':
            return 'anything';
        case 'p':
            return 'pointer';
        default:
            return value;
    }
};
const assertTypeArgument = (value) => {
    if (value === 'pointer') {
        throw new ValidationError(`"pointer" not supported (yet)`);
    }
    else if (!TYPE_ARGUMENTS.includes(value)) {
        throw new ValidationError(`invalid type ${value}`);
    }
    return value;
};
const renderMessageTransfer = (typeArgument, msgVariableName, index) => {
    switch (typeArgument) {
        case 'float':
            return `msg_floats([messageTokenToFloat(${msgVariableName}, ${index})])`;
        case 'bang':
            return `msg_bang()`;
        case 'symbol':
            return `msg_strings([messageTokenToString(${msgVariableName}, ${index})])`;
        case 'list':
        case 'anything':
            return `${msgVariableName}`;
        default:
            throw new Error(`type argument ${typeArgument} not supported (yet)`);
    }
};
const messageTokenToFloat = () => Func('messageTokenToFloat', [
    Var('Message', 'm'),
    Var('Int', 'i')
], 'Float') `
        if (msg_isFloatToken(m, i)) {
            return msg_readFloatToken(m, i)
        } else {
            return 0
        }
    `;
const messageTokenToString = () => Func('messageTokenToString', [
    Var('Message', 'm'),
    Var('Int', 'i')
], 'string') `
        if (msg_isStringToken(m, i)) {
            ${ConstVar('string', 'str', 'msg_readStringToken(m, i)')}
            if (str === 'bang') {
                return 'symbol'
            } else {
                return str
            }
        } else {
            return 'float'
        }
    `;

export { assertTypeArgument, messageTokenToFloat, messageTokenToString, renderMessageTransfer, resolveTypeArgumentAlias };
