/**
 * Helper to assert that two given AST functions have the same signature.
 */
const assertFuncSignatureEqual = (actual, expected) => {
    if (typeof actual !== 'object' || actual.astType !== 'Func') {
        throw new Error(`Expected an ast Func, got : ${actual}`);
    }
    else if (actual.args.length !== expected.args.length ||
        actual.args.some((arg, i) => {
            const expectedArg = expected.args[i];
            return !expectedArg || arg.type !== expectedArg.type;
        }) ||
        actual.returnType !== expected.returnType) {
        throw new Error(`Func should be have signature ${_printFuncSignature(expected)}` +
            ` got instead ${_printFuncSignature(actual)}`);
    }
    return actual;
};
const _printFuncSignature = (func) => `(${func.args.map((arg) => `${arg.name}: ${arg.type}`).join(', ')}) => ${func.returnType}`;

export { assertFuncSignatureEqual };
