const STATE_CLASS_NAME = 'State';
const precompileStateClass = ({ graph, settings, variableNamesAssigner, precompiledCodeAssigner, }, nodeType) => {
    const precompiledImplementation = precompiledCodeAssigner.nodeImplementations[nodeType];
    if (precompiledImplementation.nodeImplementation.state) {
        const sampleNode = Object.values(graph).find((node) => node.type === nodeType);
        if (!sampleNode) {
            throw new Error(`No node of type "${nodeType}" exists in the graph.`);
        }
        const namespaceAssigner = variableNamesAssigner.nodeImplementations[nodeType];
        const astClass = precompiledImplementation.nodeImplementation.state({
            globs: variableNamesAssigner.globs,
            node: sampleNode,
            settings,
            ns: namespaceAssigner,
        });
        precompiledImplementation.stateClass = {
            ...astClass,
            // Reset member values which are irrelevant in the state class.
            members: astClass.members.map((member) => ({
                ...member,
                value: undefined,
            })),
        };
    }
};
const precompileCore = ({ settings, variableNamesAssigner, precompiledCodeAssigner, }, nodeType) => {
    const precompiledImplementation = precompiledCodeAssigner.nodeImplementations[nodeType];
    const nodeImplementation = precompiledImplementation.nodeImplementation;
    if (nodeImplementation.core) {
        const namespaceAssigner = variableNamesAssigner.nodeImplementations[nodeType];
        precompiledImplementation.core = nodeImplementation.core({
            settings,
            globs: variableNamesAssigner.globs,
            ns: namespaceAssigner,
        });
    }
};

export { STATE_CLASS_NAME, precompileCore, precompileStateClass };
