/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
const getNode = (graph, nodeId) => {
    const node = graph[nodeId];
    if (node) {
        return node;
    }
    throw new Error(`Node "${nodeId}" not found in graph`);
};
const getInlet = (node, inletId) => {
    const inlet = node.inlets[inletId];
    if (inlet) {
        return inlet;
    }
    throw new Error(`Inlet "${inletId}" not found in node ${node.id} of type ${node.type}`);
};
const getOutlet = (node, outletId) => {
    const outlet = node.outlets[outletId];
    if (outlet) {
        return outlet;
    }
    throw new Error(`Outlet "${outletId}" not found in node ${node.id} of type ${node.type}`);
};
/** Returns the list of sinks for the outlet or an empty list. */
const getSinks = (node, outletId) => node.sinks[outletId] || [];
/** Returns the list of sources for the inlet or an empty list. */
const getSources = (node, inletId) => node.sources[inletId] || [];

export { getInlet, getNode, getOutlet, getSinks, getSources };
