import { readTypedArray, lowerString, liftString } from './core-bindings.js';

/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
const liftMessage = (wasmExports, messagePointer) => {
    const messageTokenTypesPointer = wasmExports.x_msg_getTokenTypes(messagePointer);
    const messageTokenTypes = readTypedArray(wasmExports, Int32Array, messageTokenTypesPointer);
    const message = [];
    messageTokenTypes.forEach((tokenType, tokenIndex) => {
        if (tokenType === wasmExports.MSG_FLOAT_TOKEN.valueOf()) {
            message.push(wasmExports.msg_readFloatToken(messagePointer, tokenIndex));
        }
        else if (tokenType === wasmExports.MSG_STRING_TOKEN.valueOf()) {
            const stringPointer = wasmExports.msg_readStringToken(messagePointer, tokenIndex);
            message.push(liftString(wasmExports, stringPointer));
        }
    });
    return message;
};
const lowerMessage = (wasmExports, message) => {
    const template = message.reduce((template, value) => {
        if (typeof value === 'number') {
            template.push(wasmExports.MSG_FLOAT_TOKEN.valueOf());
        }
        else if (typeof value === 'string') {
            template.push(wasmExports.MSG_STRING_TOKEN.valueOf());
            template.push(value.length);
        }
        else {
            throw new Error(`invalid message value ${value}`);
        }
        return template;
    }, []);
    // Here we should ideally pass an array of Int, but I am not sure how
    // to lower a typed array in a generic manner, so using the available bindings from `commons`.
    const templateArrayPointer = wasmExports.x_msg_createTemplate(template.length);
    const loweredTemplateArray = readTypedArray(wasmExports, Int32Array, templateArrayPointer);
    loweredTemplateArray.set(template);
    const messagePointer = wasmExports.x_msg_create(templateArrayPointer);
    message.forEach((value, index) => {
        if (typeof value === 'number') {
            wasmExports.msg_writeFloatToken(messagePointer, index, value);
        }
        else if (typeof value === 'string') {
            const stringPointer = lowerString(wasmExports, value);
            wasmExports.msg_writeStringToken(messagePointer, index, stringPointer);
        }
    });
    return messagePointer;
};

export { liftMessage, lowerMessage };
