import { liftString } from './core-bindings.js';

/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
// REF : Assemblyscript ESM bindings
const instantiateWasmModule = async (wasmBuffer, wasmImports = {}) => {
    const instanceAndModule = await WebAssembly.instantiate(wasmBuffer, {
        env: {
            abort: (messagePointer, 
            // filename, not useful because we compile everything to a single string
            _, lineNumber, columnNumber) => {
                const message = liftString(wasmExports, messagePointer);
                lineNumber = lineNumber;
                columnNumber = columnNumber;
                (() => {
                    // @external.js
                    throw Error(`${message} at ${lineNumber}:${columnNumber}`);
                })();
            },
            seed: () => {
                return (() => {
                    return Date.now() * Math.random();
                })();
            },
            'console.log': (textPointer) => {
                console.log(liftString(wasmExports, textPointer));
            },
        },
        ...wasmImports,
    });
    const wasmExports = instanceAndModule.instance
        .exports;
    return instanceAndModule.instance;
};

export { instantiateWasmModule };
