import { createModule, getFloatArrayType } from '../../run/run-helpers.js';

/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
const createRawModule = (code) => new Function(`
        ${code}
        return exports
    `)();
const createBindings = (rawModule) => ({
    fs: { type: 'proxy', value: createFsModule(rawModule) },
    metadata: { type: 'raw' },
    initialize: { type: 'raw' },
    dspLoop: { type: 'raw' },
    io: { type: 'raw' },
    commons: {
        type: 'proxy',
        value: createCommonsModule(rawModule, rawModule.metadata.audioSettings.bitDepth),
    },
});
const createEngine = (code) => {
    const rawModule = createRawModule(code);
    return createModule(rawModule, createBindings(rawModule));
};
const createFsModule = (rawModule) => {
    const fs = createModule(rawModule, {
        onReadSoundFile: { type: 'callback', value: () => undefined },
        onWriteSoundFile: { type: 'callback', value: () => undefined },
        onOpenSoundReadStream: { type: 'callback', value: () => undefined },
        onOpenSoundWriteStream: { type: 'callback', value: () => undefined },
        onSoundStreamData: { type: 'callback', value: () => undefined },
        onCloseSoundStream: { type: 'callback', value: () => undefined },
        sendReadSoundFileResponse: {
            type: 'proxy',
            value: rawModule.x_fs_onReadSoundFileResponse,
        },
        sendWriteSoundFileResponse: {
            type: 'proxy',
            value: rawModule.x_fs_onWriteSoundFileResponse,
        },
        sendSoundStreamData: {
            type: 'proxy',
            value: rawModule.x_fs_onSoundStreamData,
        },
        closeSoundStream: {
            type: 'proxy',
            value: rawModule.x_fs_onCloseSoundStream,
        },
    });
    rawModule.i_fs_openSoundWriteStream = (...args) => fs.onOpenSoundWriteStream(...args);
    rawModule.i_fs_sendSoundStreamData = (...args) => fs.onSoundStreamData(...args);
    rawModule.i_fs_openSoundReadStream = (...args) => fs.onOpenSoundReadStream(...args);
    rawModule.i_fs_closeSoundStream = (...args) => fs.onCloseSoundStream(...args);
    rawModule.i_fs_writeSoundFile = (...args) => fs.onWriteSoundFile(...args);
    rawModule.i_fs_readSoundFile = (...args) => fs.onReadSoundFile(...args);
    return fs;
};
const createCommonsModule = (rawModule, bitDepth) => {
    const floatArrayType = getFloatArrayType(bitDepth);
    return createModule(rawModule, {
        getArray: { type: 'proxy', value: rawModule.commons_getArray },
        setArray: {
            type: 'proxy',
            value: (arrayName, array) => rawModule.commons_setArray(arrayName, new floatArrayType(array)),
        },
    });
};

export { createBindings, createEngine, createRawModule };
