/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
/** Generate an integer series from 0 to `count` (non-inclusive). */
const countTo = (count) => {
    const results = [];
    for (let i = 0; i < count; i++) {
        results.push(i);
    }
    return results;
};
/**
 * @param func Called for each element in `src`. Returns a pair `[<key>, <value>]`.
 * @returns A new object whoses keys and values are the result of
 * applying `func` to each element in `src`.
 */
const mapArray = (src, func) => {
    const dest = {};
    src.forEach((srcValue, i) => {
        const [key, destValue] = func(srcValue, i);
        dest[key] = destValue;
    });
    return dest;
};
/**
 * Renders one of several alternative bits of code.
 *
 * @param routes A list of alternatives `[<test>, <code>]`
 * @returns The first `code` whose `test` evaluated to true.
 */
const renderSwitch = (...routes) => {
    const matchedRoute = routes.find(([test]) => test);
    if (!matchedRoute) {
        throw new Error(`no route found`);
    }
    return matchedRoute[1];
};

export { countTo, mapArray, renderSwitch };
