var ASSEMBLY_SCRIPT_WASM_BINDINGS_CODE = "var AssemblyScriptWasmBindings = (function (exports) {\n    'use strict';\n\n    const getFloatArrayType = (bitDepth) => bitDepth === 64 ? Float64Array : Float32Array;\n    const createModule = (rawModule, bindings) => new Proxy({}, {\n        get: (_, k) => {\n            if (bindings.hasOwnProperty(k)) {\n                const key = String(k);\n                const bindingSpec = bindings[key];\n                switch (bindingSpec.type) {\n                    case 'raw':\n                        if (k in rawModule) {\n                            return rawModule[key];\n                        }\n                        else {\n                            throw new Error(`Key ${String(key)} doesn't exist in raw module`);\n                        }\n                    case 'proxy':\n                    case 'callback':\n                        return bindingSpec.value;\n                }\n            }\n            else {\n                return undefined;\n            }\n        },\n        set: (_, k, newValue) => {\n            if (bindings.hasOwnProperty(String(k))) {\n                const key = String(k);\n                const bindingSpec = bindings[key];\n                if (bindingSpec.type === 'callback') {\n                    bindingSpec.value = newValue;\n                }\n                else {\n                    throw new Error(`Binding key ${String(key)} is read-only`);\n                }\n            }\n            else {\n                throw new Error(`Key ${String(k)} is not defined in bindings`);\n            }\n            return true;\n        },\n    });\n\n    const liftString = (wasmExports, pointer) => {\n        if (!pointer) {\n            throw new Error('Cannot lift a null pointer');\n        }\n        pointer = pointer >>> 0;\n        const end = (pointer +\n            new Uint32Array(wasmExports.memory.buffer)[(pointer - 4) >>> 2]) >>>\n            1;\n        const memoryU16 = new Uint16Array(wasmExports.memory.buffer);\n        let start = pointer >>> 1;\n        let string = '';\n        while (end - start > 1024) {\n            string += String.fromCharCode(...memoryU16.subarray(start, (start += 1024)));\n        }\n        return string + String.fromCharCode(...memoryU16.subarray(start, end));\n    };\n    const lowerString = (wasmExports, value) => {\n        if (value == null) {\n            throw new Error('Cannot lower a null string');\n        }\n        const length = value.length, pointer = wasmExports.__new(length << 1, 1) >>> 0, memoryU16 = new Uint16Array(wasmExports.memory.buffer);\n        for (let i = 0; i < length; ++i)\n            memoryU16[(pointer >>> 1) + i] = value.charCodeAt(i);\n        return pointer;\n    };\n    const readTypedArray = (wasmExports, constructor, pointer) => {\n        if (!pointer) {\n            throw new Error('Cannot lift a null pointer');\n        }\n        const memoryU32 = new Uint32Array(wasmExports.memory.buffer);\n        return new constructor(wasmExports.memory.buffer, memoryU32[(pointer + 4) >>> 2], memoryU32[(pointer + 8) >>> 2] / constructor.BYTES_PER_ELEMENT);\n    };\n    const lowerFloatArray = (wasmExports, bitDepth, data) => {\n        const arrayType = getFloatArrayType(bitDepth);\n        const arrayPointer = wasmExports.createFloatArray(data.length);\n        const array = readTypedArray(wasmExports, arrayType, arrayPointer);\n        array.set(data);\n        return { array, arrayPointer };\n    };\n    const lowerListOfFloatArrays = (wasmExports, bitDepth, data) => {\n        const arraysPointer = wasmExports.x_core_createListOfArrays();\n        data.forEach((array) => {\n            const { arrayPointer } = lowerFloatArray(wasmExports, bitDepth, array);\n            wasmExports.x_core_pushToListOfArrays(arraysPointer, arrayPointer);\n        });\n        return arraysPointer;\n    };\n    const readListOfFloatArrays = (wasmExports, bitDepth, listOfArraysPointer) => {\n        const listLength = wasmExports.x_core_getListOfArraysLength(listOfArraysPointer);\n        const arrays = [];\n        const arrayType = getFloatArrayType(bitDepth);\n        for (let i = 0; i < listLength; i++) {\n            const arrayPointer = wasmExports.x_core_getListOfArraysElem(listOfArraysPointer, i);\n            arrays.push(readTypedArray(wasmExports, arrayType, arrayPointer));\n        }\n        return arrays;\n    };\n\n    const liftMessage = (wasmExports, messagePointer) => {\n        const messageTokenTypesPointer = wasmExports.x_msg_getTokenTypes(messagePointer);\n        const messageTokenTypes = readTypedArray(wasmExports, Int32Array, messageTokenTypesPointer);\n        const message = [];\n        messageTokenTypes.forEach((tokenType, tokenIndex) => {\n            if (tokenType === wasmExports.MSG_FLOAT_TOKEN.valueOf()) {\n                message.push(wasmExports.msg_readFloatToken(messagePointer, tokenIndex));\n            }\n            else if (tokenType === wasmExports.MSG_STRING_TOKEN.valueOf()) {\n                const stringPointer = wasmExports.msg_readStringToken(messagePointer, tokenIndex);\n                message.push(liftString(wasmExports, stringPointer));\n            }\n        });\n        return message;\n    };\n    const lowerMessage = (wasmExports, message) => {\n        const template = message.reduce((template, value) => {\n            if (typeof value === 'number') {\n                template.push(wasmExports.MSG_FLOAT_TOKEN.valueOf());\n            }\n            else if (typeof value === 'string') {\n                template.push(wasmExports.MSG_STRING_TOKEN.valueOf());\n                template.push(value.length);\n            }\n            else {\n                throw new Error(`invalid message value ${value}`);\n            }\n            return template;\n        }, []);\n        const templateArrayPointer = wasmExports.x_msg_createTemplate(template.length);\n        const loweredTemplateArray = readTypedArray(wasmExports, Int32Array, templateArrayPointer);\n        loweredTemplateArray.set(template);\n        const messagePointer = wasmExports.x_msg_create(templateArrayPointer);\n        message.forEach((value, index) => {\n            if (typeof value === 'number') {\n                wasmExports.msg_writeFloatToken(messagePointer, index, value);\n            }\n            else if (typeof value === 'string') {\n                const stringPointer = lowerString(wasmExports, value);\n                wasmExports.msg_writeStringToken(messagePointer, index, stringPointer);\n            }\n        });\n        return messagePointer;\n    };\n\n    const mapArray = (src, func) => {\n        const dest = {};\n        src.forEach((srcValue, i) => {\n            const [key, destValue] = func(srcValue, i);\n            dest[key] = destValue;\n        });\n        return dest;\n    };\n\n    const instantiateWasmModule = async (wasmBuffer, wasmImports = {}) => {\n        const instanceAndModule = await WebAssembly.instantiate(wasmBuffer, {\n            env: {\n                abort: (messagePointer, _, lineNumber, columnNumber) => {\n                    const message = liftString(wasmExports, messagePointer);\n                    lineNumber = lineNumber;\n                    columnNumber = columnNumber;\n                    (() => {\n                        throw Error(`${message} at ${lineNumber}:${columnNumber}`);\n                    })();\n                },\n                seed: () => {\n                    return (() => {\n                        return Date.now() * Math.random();\n                    })();\n                },\n                'console.log': (textPointer) => {\n                    console.log(liftString(wasmExports, textPointer));\n                },\n            },\n            ...wasmImports,\n        });\n        const wasmExports = instanceAndModule.instance\n            .exports;\n        return instanceAndModule.instance;\n    };\n\n    const updateWasmInOuts = (rawModule, engineData) => {\n        engineData.wasmOutput = readTypedArray(rawModule, engineData.arrayType, rawModule.getOutput());\n        engineData.wasmInput = readTypedArray(rawModule, engineData.arrayType, rawModule.getInput());\n    };\n    const createEngineLifecycleBindings = (rawModule, engineData) => {\n        return {\n            initialize: {\n                type: 'proxy',\n                value: (sampleRate, blockSize) => {\n                    engineData.metadata.audioSettings.blockSize = blockSize;\n                    engineData.metadata.audioSettings.sampleRate = sampleRate;\n                    engineData.blockSize = blockSize;\n                    rawModule.initialize(sampleRate, blockSize);\n                    updateWasmInOuts(rawModule, engineData);\n                },\n            },\n            dspLoop: {\n                type: 'proxy',\n                value: (input, output) => {\n                    for (let channel = 0; channel < input.length; channel++) {\n                        engineData.wasmInput.set(input[channel], channel * engineData.blockSize);\n                    }\n                    updateWasmInOuts(rawModule, engineData);\n                    rawModule.dspLoop();\n                    updateWasmInOuts(rawModule, engineData);\n                    for (let channel = 0; channel < output.length; channel++) {\n                        output[channel].set(engineData.wasmOutput.subarray(engineData.blockSize * channel, engineData.blockSize * (channel + 1)));\n                    }\n                },\n            },\n        };\n    };\n    const createIoMessageReceiversBindings = (rawModule, engineData) => Object.entries(engineData.metadata.compilation.io.messageReceivers).reduce((bindings, [nodeId, spec]) => ({\n        ...bindings,\n        [nodeId]: {\n            type: 'proxy',\n            value: mapArray(spec.portletIds, (inletId) => [\n                inletId,\n                (message) => {\n                    const messagePointer = lowerMessage(rawModule, message);\n                    rawModule[engineData.metadata.compilation.variableNamesIndex\n                        .io.messageReceivers[nodeId][inletId]](messagePointer);\n                },\n            ]),\n        },\n    }), {});\n    const createIoMessageSendersBindings = (_, engineData) => Object.entries(engineData.metadata.compilation.io.messageSenders).reduce((bindings, [nodeId, spec]) => ({\n        ...bindings,\n        [nodeId]: {\n            type: 'proxy',\n            value: mapArray(spec.portletIds, (outletId) => [\n                outletId,\n                {\n                    onMessage: () => undefined,\n                },\n            ]),\n        },\n    }), {});\n    const ioMsgSendersImports = (forwardReferences, metadata) => {\n        const wasmImports = {};\n        const { variableNamesIndex } = metadata.compilation;\n        Object.entries(metadata.compilation.io.messageSenders).forEach(([nodeId, spec]) => {\n            spec.portletIds.forEach((outletId) => {\n                const listenerName = variableNamesIndex.io.messageSenders[nodeId][outletId];\n                wasmImports[listenerName] = (messagePointer) => {\n                    const message = liftMessage(forwardReferences.rawModule, messagePointer);\n                    forwardReferences.modules.io.messageSenders[nodeId][outletId].onMessage(message);\n                };\n            });\n        });\n        return wasmImports;\n    };\n    const readMetadata = async (wasmBuffer) => {\n        const inputImports = {};\n        const wasmModule = WebAssembly.Module.imports(new WebAssembly.Module(wasmBuffer));\n        wasmModule\n            .filter((imprt) => imprt.module === 'input' && imprt.kind === 'function')\n            .forEach((imprt) => (inputImports[imprt.name] = () => undefined));\n        const wasmInstance = await instantiateWasmModule(wasmBuffer, {\n            input: inputImports,\n        });\n        const wasmExports = wasmInstance.exports;\n        const stringPointer = wasmExports.metadata.valueOf();\n        const metadataJSON = liftString(wasmExports, stringPointer);\n        return JSON.parse(metadataJSON);\n    };\n\n    const createFsBindings = (rawModule, engineData) => ({\n        sendReadSoundFileResponse: {\n            type: 'proxy',\n            value: (operationId, status, sound) => {\n                let soundPointer = 0;\n                if (sound) {\n                    soundPointer = lowerListOfFloatArrays(rawModule, engineData.bitDepth, sound);\n                }\n                rawModule.x_fs_onReadSoundFileResponse(operationId, status, soundPointer);\n                updateWasmInOuts(rawModule, engineData);\n            },\n        },\n        sendWriteSoundFileResponse: {\n            type: 'proxy',\n            value: rawModule.x_fs_onWriteSoundFileResponse,\n        },\n        sendSoundStreamData: {\n            type: 'proxy',\n            value: (operationId, sound) => {\n                const soundPointer = lowerListOfFloatArrays(rawModule, engineData.bitDepth, sound);\n                const writtenFrameCount = rawModule.x_fs_onSoundStreamData(operationId, soundPointer);\n                updateWasmInOuts(rawModule, engineData);\n                return writtenFrameCount;\n            },\n        },\n        closeSoundStream: {\n            type: 'proxy',\n            value: rawModule.x_fs_onCloseSoundStream,\n        },\n        onReadSoundFile: { type: 'callback', value: () => undefined },\n        onWriteSoundFile: { type: 'callback', value: () => undefined },\n        onOpenSoundReadStream: { type: 'callback', value: () => undefined },\n        onOpenSoundWriteStream: { type: 'callback', value: () => undefined },\n        onSoundStreamData: { type: 'callback', value: () => undefined },\n        onCloseSoundStream: { type: 'callback', value: () => undefined },\n    });\n    const createFsImports = (forwardReferences) => {\n        let wasmImports = {\n            i_fs_readSoundFile: (operationId, urlPointer, infoPointer) => {\n                const url = liftString(forwardReferences.rawModule, urlPointer);\n                const info = liftMessage(forwardReferences.rawModule, infoPointer);\n                forwardReferences.modules.fs.onReadSoundFile(operationId, url, info);\n            },\n            i_fs_writeSoundFile: (operationId, soundPointer, urlPointer, infoPointer) => {\n                const sound = readListOfFloatArrays(forwardReferences.rawModule, forwardReferences.engineData.bitDepth, soundPointer);\n                const url = liftString(forwardReferences.rawModule, urlPointer);\n                const info = liftMessage(forwardReferences.rawModule, infoPointer);\n                forwardReferences.modules.fs.onWriteSoundFile(operationId, sound, url, info);\n            },\n            i_fs_openSoundReadStream: (operationId, urlPointer, infoPointer) => {\n                const url = liftString(forwardReferences.rawModule, urlPointer);\n                const info = liftMessage(forwardReferences.rawModule, infoPointer);\n                updateWasmInOuts(forwardReferences.rawModule, forwardReferences.engineData);\n                forwardReferences.modules.fs.onOpenSoundReadStream(operationId, url, info);\n            },\n            i_fs_openSoundWriteStream: (operationId, urlPointer, infoPointer) => {\n                const url = liftString(forwardReferences.rawModule, urlPointer);\n                const info = liftMessage(forwardReferences.rawModule, infoPointer);\n                forwardReferences.modules.fs.onOpenSoundWriteStream(operationId, url, info);\n            },\n            i_fs_sendSoundStreamData: (operationId, blockPointer) => {\n                const block = readListOfFloatArrays(forwardReferences.rawModule, forwardReferences.engineData.bitDepth, blockPointer);\n                forwardReferences.modules.fs.onSoundStreamData(operationId, block);\n            },\n            i_fs_closeSoundStream: (...args) => forwardReferences.modules.fs.onCloseSoundStream(...args),\n        };\n        return wasmImports;\n    };\n\n    const createCommonsBindings = (rawModule, engineData) => ({\n        getArray: {\n            type: 'proxy',\n            value: (arrayName) => {\n                const arrayNamePointer = lowerString(rawModule, arrayName);\n                const arrayPointer = rawModule.commons_getArray(arrayNamePointer);\n                return readTypedArray(rawModule, engineData.arrayType, arrayPointer);\n            },\n        },\n        setArray: {\n            type: 'proxy',\n            value: (arrayName, array) => {\n                const stringPointer = lowerString(rawModule, arrayName);\n                const { arrayPointer } = lowerFloatArray(rawModule, engineData.bitDepth, array);\n                rawModule.commons_setArray(stringPointer, arrayPointer);\n                updateWasmInOuts(rawModule, engineData);\n            },\n        },\n    });\n\n    const createEngine = async (wasmBuffer) => {\n        const { rawModule, engineData, forwardReferences } = await createRawModule(wasmBuffer);\n        const engineBindings = await createBindings(rawModule, engineData, forwardReferences);\n        return createModule(rawModule, engineBindings);\n    };\n    const createRawModule = async (wasmBuffer) => {\n        const metadata = await readMetadata(wasmBuffer);\n        const forwardReferences = { modules: {} };\n        const wasmImports = {\n            ...createFsImports(forwardReferences),\n            ...ioMsgSendersImports(forwardReferences, metadata),\n        };\n        const bitDepth = metadata.audioSettings.bitDepth;\n        const arrayType = getFloatArrayType(bitDepth);\n        const engineData = {\n            metadata,\n            wasmOutput: new arrayType(0),\n            wasmInput: new arrayType(0),\n            arrayType,\n            bitDepth,\n            blockSize: 0,\n        };\n        const wasmInstance = await instantiateWasmModule(wasmBuffer, {\n            input: wasmImports,\n        });\n        const rawModule = wasmInstance.exports;\n        return { rawModule, engineData, forwardReferences };\n    };\n    const createBindings = async (rawModule, engineData, forwardReferences) => {\n        const commons = createModule(rawModule, createCommonsBindings(rawModule, engineData));\n        const fs = createModule(rawModule, createFsBindings(rawModule, engineData));\n        const io = {\n            messageReceivers: createModule(rawModule, createIoMessageReceiversBindings(rawModule, engineData)),\n            messageSenders: createModule(rawModule, createIoMessageSendersBindings(rawModule, engineData)),\n        };\n        forwardReferences.modules.fs = fs;\n        forwardReferences.modules.io = io;\n        forwardReferences.engineData = engineData;\n        forwardReferences.rawModule = rawModule;\n        return {\n            ...createEngineLifecycleBindings(rawModule, engineData),\n            metadata: { type: 'proxy', value: engineData.metadata },\n            commons: { type: 'proxy', value: commons },\n            fs: { type: 'proxy', value: fs },\n            io: { type: 'proxy', value: io },\n        };\n    };\n\n    exports.createBindings = createBindings;\n    exports.createEngine = createEngine;\n    exports.createRawModule = createRawModule;\n\n    return exports;\n\n})({});\n";

export { ASSEMBLY_SCRIPT_WASM_BINDINGS_CODE as default };
