var JAVA_SCRIPT_BINDINGS_CODE = "var JavaScriptBindings = (function (exports) {\n    'use strict';\n\n    const getFloatArrayType = (bitDepth) => bitDepth === 64 ? Float64Array : Float32Array;\n    const createModule = (rawModule, bindings) => new Proxy({}, {\n        get: (_, k) => {\n            if (bindings.hasOwnProperty(k)) {\n                const key = String(k);\n                const bindingSpec = bindings[key];\n                switch (bindingSpec.type) {\n                    case 'raw':\n                        if (k in rawModule) {\n                            return rawModule[key];\n                        }\n                        else {\n                            throw new Error(`Key ${String(key)} doesn't exist in raw module`);\n                        }\n                    case 'proxy':\n                    case 'callback':\n                        return bindingSpec.value;\n                }\n            }\n            else {\n                return undefined;\n            }\n        },\n        set: (_, k, newValue) => {\n            if (bindings.hasOwnProperty(String(k))) {\n                const key = String(k);\n                const bindingSpec = bindings[key];\n                if (bindingSpec.type === 'callback') {\n                    bindingSpec.value = newValue;\n                }\n                else {\n                    throw new Error(`Binding key ${String(key)} is read-only`);\n                }\n            }\n            else {\n                throw new Error(`Key ${String(k)} is not defined in bindings`);\n            }\n            return true;\n        },\n    });\n\n    const createRawModule = (code) => new Function(`\n        ${code}\n        return exports\n    `)();\n    const createBindings = (rawModule) => ({\n        fs: { type: 'proxy', value: createFsModule(rawModule) },\n        metadata: { type: 'raw' },\n        initialize: { type: 'raw' },\n        dspLoop: { type: 'raw' },\n        io: { type: 'raw' },\n        commons: {\n            type: 'proxy',\n            value: createCommonsModule(rawModule, rawModule.metadata.audioSettings.bitDepth),\n        },\n    });\n    const createEngine = (code) => {\n        const rawModule = createRawModule(code);\n        return createModule(rawModule, createBindings(rawModule));\n    };\n    const createFsModule = (rawModule) => {\n        const fs = createModule(rawModule, {\n            onReadSoundFile: { type: 'callback', value: () => undefined },\n            onWriteSoundFile: { type: 'callback', value: () => undefined },\n            onOpenSoundReadStream: { type: 'callback', value: () => undefined },\n            onOpenSoundWriteStream: { type: 'callback', value: () => undefined },\n            onSoundStreamData: { type: 'callback', value: () => undefined },\n            onCloseSoundStream: { type: 'callback', value: () => undefined },\n            sendReadSoundFileResponse: {\n                type: 'proxy',\n                value: rawModule.x_fs_onReadSoundFileResponse,\n            },\n            sendWriteSoundFileResponse: {\n                type: 'proxy',\n                value: rawModule.x_fs_onWriteSoundFileResponse,\n            },\n            sendSoundStreamData: {\n                type: 'proxy',\n                value: rawModule.x_fs_onSoundStreamData,\n            },\n            closeSoundStream: {\n                type: 'proxy',\n                value: rawModule.x_fs_onCloseSoundStream,\n            },\n        });\n        rawModule.i_fs_openSoundWriteStream = (...args) => fs.onOpenSoundWriteStream(...args);\n        rawModule.i_fs_sendSoundStreamData = (...args) => fs.onSoundStreamData(...args);\n        rawModule.i_fs_openSoundReadStream = (...args) => fs.onOpenSoundReadStream(...args);\n        rawModule.i_fs_closeSoundStream = (...args) => fs.onCloseSoundStream(...args);\n        rawModule.i_fs_writeSoundFile = (...args) => fs.onWriteSoundFile(...args);\n        rawModule.i_fs_readSoundFile = (...args) => fs.onReadSoundFile(...args);\n        return fs;\n    };\n    const createCommonsModule = (rawModule, bitDepth) => {\n        const floatArrayType = getFloatArrayType(bitDepth);\n        return createModule(rawModule, {\n            getArray: { type: 'proxy', value: rawModule.commons_getArray },\n            setArray: {\n                type: 'proxy',\n                value: (arrayName, array) => rawModule.commons_setArray(arrayName, new floatArrayType(array)),\n            },\n        });\n    };\n\n    exports.createBindings = createBindings;\n    exports.createEngine = createEngine;\n    exports.createRawModule = createRawModule;\n\n    return exports;\n\n})({});\n";

export { JAVA_SCRIPT_BINDINGS_CODE as default };
